#!/usr/bin/env node
"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const _1 = require(".");
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const update_notifier_1 = __importDefault(require("update-notifier"));
// import pkg from '../package.json'
const yargs_1 = __importDefault(require("yargs"));
const yargs_parser_1 = __importDefault(require("yargs-parser"));
const pkg = _1.Utils.loadCorePackageInfo();
update_notifier_1.default({ pkg, updateCheckInterval: 1000 * 60 * 60 * 24 * 7 }).notify({
    defer: false,
    isGlobal: true
});
let parsedArgv = yargs_parser_1.default(process.argv.slice(2));
let appConfig = _1.Utils.getApplicationConfig();
yargs_1.default.config(appConfig);
parsedArgv = _1.Utils._.merge(appConfig, parsedArgv);
const cache = _1.Utils.getInternalCache();
cache.set('argv', parsedArgv);
const plugins = _1.Utils.getAllPluginsMapping();
const config = _1.Utils.getCombinedConfig();
const packageConfig = _1.Utils.loadPackageInfo();
if (!parsedArgv.scriptName) {
    yargs_1.default.hide('script-name').option('script-name', {
        default: 'zignis',
        describe: 'Rename script name.',
        type: 'string'
    });
}
else {
    if (!_1.Utils._.isString(parsedArgv.scriptName)) {
        _1.Utils.error('--script-name must be string, should be used only once.');
    }
    yargs_1.default.scriptName(parsedArgv.scriptName);
}
yargs_1.default.hide('plugin-prefix').option('plugin-prefix', {
    default: 'zignis',
    type: 'array',
    describe: 'Set plugin prefix.'
});
let scriptName = parsedArgv.scriptName || 'zignis';
if (!parsedArgv.disableCoreCommand) {
    // Load local commands
    if (packageConfig.name !== scriptName) {
        yargs_1.default.commandDir('commands');
    }
    else if (appConfig.commandDir && fs_1.default.existsSync(path_1.default.resolve(process.cwd(), appConfig.commandDir))) {
        yargs_1.default.commandDir(path_1.default.resolve(process.cwd(), appConfig.commandDir));
    }
}
// Load plugin commands
if (plugins) {
    Object.keys(plugins).map(function (plugin) {
        if (config.pluginConfigs[plugin].commandDir && fs_1.default.existsSync(path_1.default.resolve(plugins[plugin], config.pluginConfigs[plugin].commandDir))) {
            yargs_1.default.commandDir(path_1.default.resolve(plugins[plugin], config.pluginConfigs[plugin].commandDir));
        }
    });
}
// Load application commands
if (packageConfig.name !== scriptName &&
    appConfig.commandDir &&
    fs_1.default.existsSync(path_1.default.resolve(process.cwd(), appConfig.commandDir))) {
    yargs_1.default.commandDir(path_1.default.resolve(process.cwd(), appConfig.commandDir));
}
;
(() => __awaiter(this, void 0, void 0, function* () {
    try {
        if (!parsedArgv.getYargsCompletions) {
            let beforeHooks = yield _1.Utils.invokeHook('beforeCommand');
            Object.keys(beforeHooks).map(function (hook) {
                beforeHooks[hook](parsedArgv, yargs_1.default);
            });
        }
        if (!parsedArgv.disableCoreCommand && !parsedArgv.disableCore) {
            yargs_1.default.hide('disable-core-command').option('disable-core-command', {
                alias: 'disable-core',
                describe: 'Disable core commands.'
            });
        }
        if (!parsedArgv.disableGlobalPlugin && !parsedArgv.disableGlobalPlugins) {
            yargs_1.default.hide('disable-global-plugin').option('disable-global-plugin', {
                alias: 'disable-global-plugins',
                describe: 'Disable global plugins.'
            });
        }
        if (!parsedArgv.disableHomePlugin && !parsedArgv.disableHomePlugins) {
            yargs_1.default.hide('disable-home-plugin').option('disable-home-plugin', {
                alias: 'disable-home-plugins',
                describe: 'Disable home plugins.'
            });
        }
        if (!parsedArgv.disableCompletionCommand && !parsedArgv.disableCompletion) {
            yargs_1.default.hide('disable-completion-command').option('disable-completion-command', {
                alias: 'disable-completion',
                describe: 'Disable completion command.'
            });
            if (!parsedArgv.hideCompletionCommand && !parsedArgv.hideCompletion) {
                yargs_1.default.hide('hide-completion-command').option('hide-completion-command', {
                    alias: 'hide-completion',
                    describe: 'Hide completion command.'
                });
                yargs_1.default.completion('completion', 'Generate completion script');
            }
            else {
                // @ts-ignore, @types/yargs type def not correct
                yargs_1.default.completion('completion', false);
            }
        }
        if (!parsedArgv.hideEpilog) {
            yargs_1.default.hide('hide-epilog').option('hide-epilog', {
                describe: 'Hide epilog.'
            });
            yargs_1.default.hide('epilog').option('epilog', {
                default: false,
                alias: 'epilog',
                describe: 'Set epilog.'
            });
        }
        !parsedArgv.hideEpilog && yargs_1.default.epilog(parsedArgv.epilog || 'Find more information at https://zignis.js.org');
        // eslint-disable-next-line
        yargs_1.default
            .help()
            .alias('h', 'help')
            .exitProcess(false)
            .recommendCommands()
            .wrap(Math.min(120, yargs_1.default.terminalWidth())).argv;
        if (!parsedArgv.getYargsCompletions) {
            let afterHooks = yield _1.Utils.invokeHook('afterCommand');
            Object.keys(afterHooks).map(function (hook) {
                afterHooks[hook](parsedArgv, yargs_1.default);
            });
        }
    }
    catch (e) {
        if (!e.name || e.name !== 'YError') {
            _1.Utils.error(e.stack);
        }
    }
}))();
